<?php

class bookingsActions extends sfActions
{
    public function executeIndex()
    {
        $this->listings = Doctrine::getTable('Listing')
            ->createQuery('l')
            ->innerJoin('l.Requests r')
            ->leftJoin('l.OwnerPickupBranch pb')
            ->leftJoin('l.OwnerDropoffBranch db')
            ->where('l.owner_id = ?', $this->getUser()->getOwnerId())
            ->addWhere('r.status = 0')
            ->orderBy('created_at desc')
            ->execute();
    }

    public function executeProcess($sfRequest)
    {
        $ids = $sfRequest->getParameter('request');
        if (!$ids && $sfRequest->getParameter('ids')) {
            $ids = explode(',', $sfRequest->getParameter('ids'));
        }

        $this->forward404Unless(count($ids));
        $this->requests = Doctrine::getTable('Request')
            ->createQuery('r')
            ->innerJoin('r.Listing l')
            ->whereIn('r.id', $ids)
            ->addWhere('l.owner_id = ?', $this->getUser()->getOwnerId())
            ->execute();

        $errors = $this->validateRequests($this->requests);
        

        if (count($errors)) {
            $this->listings = Doctrine::getTable('Listing')
                ->createQuery('l')
                ->innerJoin('l.Requests r')
                ->leftJoin('l.OwnerPickupBranch pb')
                ->leftJoin('l.OwnerDropoffBranch db')
                ->where('l.owner_id = ?', $this->getUser()->getOwnerId())
                ->addWhere('r.status = 0')
                ->orderBy('created_at desc')
                ->execute();
            $this->errors = $errors;
            $this->setTemplate('index');
        }
    }

    private function validateRequests($requests)
    {
        $errors = [];

        foreach ($this->requests as $request) {
            if (!$request->isStatus(Request::STATUS_PENDING)) {
                $errors[] = ['type' => 'checksum', 'request' =>  $request];
            }
        }

        return $errors;
    }

    public function executeConfirm($request)
    {
        $ids = $request->getParameter('request');

        if ($request->isMethod('POST')) {

            
            $requests = Doctrine::getTable('Request')
                ->createQuery('r')
                ->innerJoin('r.Listing l')
                ->where('r.status = ?', 0)
                ->whereIn('r.id', $ids)
                ->addWhere('l.owner_id = ?', $this->getUser()->getOwnerId())
                ->execute();
            $totalPrice = $this->getTotalPrice($requests);

            if ($this->getUser()->getProfile()->mustCreditAccount($totalPrice)) {
                return $this->redirect('bookings/payment?ids='.implode(',', $ids).'&amount='.$totalPrice);
            }

            foreach ($requests as  $request) {
                $request->set('booking_type', 'easy');
                $request->accept($this->getUser()->getProfile()->getUsername());
            }

            return $this->redirect('bookings/confirm?ids='.implode(',', $ids));
        }

        $this->forward404Unless($request->isMethod('GET') && $request->hasParameter('ids'));

        $requests = Doctrine::getTable('Request')
                ->createQuery('r')
                ->innerJoin('r.Listing l')
                ->whereIn('r.id', explode(',', $request->getParameter('ids')))
                ->addWhere('l.owner_id = ?', $this->getUser()->getOwnerId())
                ->execute();

        $this->requests = Transfercar\Export\BookingsExport::getRequestColumns($this->getUser()->getProfile()->get('Owner'), $requests);
        $this->headers = array_keys($this->requests[0]);
        $this->request_ids = array_map(function($request){ return $request->getId(); }, $requests->getData());
    }

    private function getTotalPrice($requests)
    {
        $total = 0;
        foreach ($requests as $request) {
            $listing = $request->get('Listing');
            $price = $listing->getTotalPrice($request);
            $total +=$price;
        }

        return $total;
    }

    public function executePayment($request)
    {
        $this->amount = $request->getParameter('amount');

        if (!$this->getUser()->getProfile()->mustCreditAccount($this->amount)) {
            $this->redirect('bookings/process?ids='.$request->getParameter('ids'));
        }
        $this->payment_info = null;
        $this->currency_name = strtoupper($this->getUser()->getUserDomain()).'D';
        $this->credits = VirtualCurrency::retreiveCreditsByCountryId($this->getUser()->getCountry());
    }

    public function executeGenerate()
    {

        $conn = Doctrine::getTable('Listing')->getConnection();
        $conn->beginTransaction();

        try
        {   
            $listing = $this->generateListing();
            $requests = $this->generateRequests($listing);
            $conn->commit();
        }
        catch (Exception $e)
        {
            $conn->rollback();
            throw $e;
        }

        return $this->redirect('bookings/index');
    }

    public function executeGeneraterequest($request)
    {
        $listing = Doctrine::getTable('Listing')->find($request->getParameter('id'));
        $this->forward404Unless($listing);
        $this->generateRequests($listing, 1);

        return $this->redirect('bookings/index');
    }

    public function executeExport($request)
    {
        $ids = $request->getParameter('ids');

        $requests = Doctrine::getTable('Request')
            ->createQuery('r')
            ->innerJoin('r.Listing l')
            ->whereIn('r.id', explode(',',$ids))
            ->addWhere('l.owner_id = ?', $this->getUser()->getOwnerId())
            ->execute();

        $output = Transfercar\Export\BookingsExport::exportCSV($this->getUser()->getProfile()->get('Owner'), $requests);

        $this->getResponse()->setHttpHeader('Content-Type', 'application/csv');
        $this->getResponse()->setHttpHeader('Content-Disposition', 'inline; filename=transfercar-export-' . date ( "Y-m-d" ) . '.csv;');
        $this->getResponse()->setHttpHeader('Cache-Control', 'private');

        return $this->renderText($output);
    }

    public function executeImport($request)
    {
        $form = new EasyBookingImportForm([], ['owner_id' => $this->getUser()->getOwnerId()]);
        if ($request->isMethod('POST')) {
            $form->bind($request->getParameter('easybook'), $request->getFiles('easybook'));

            if ($form->isValid()) {
                Transfercar\Import\BookingsImport::importReferenceNumbers($form->getValue('import'));
                return $this->redirect('bookings/index');
            }
        }

        $this->form = $form;
    }

    private function generateListing()
    {
        $branches = Doctrine::getTable('OwnerBranch')
            ->getTypesList(
                $this->getUser()->getAttribute('profile'), 
                $this->getUser()->getCountry()
            );

        $branchIdx = array_rand($branches, 2);

        $vehicles = Doctrine::getTable('OwnerVehicleType')
            ->getTypesList(
                $this->getUser()->getAttribute('profile'), 
                $this->getUser()->getCountry()
            );

        $vehicleIdx = array_rand($vehicles, 2);

        $policy = Doctrine::getTable('InsurancePolicy')
            ->createQuery()
            ->where('owner_id = ?', $this->getUser()->getOwnerId())
            ->limit(1)
            ->fetchOne();

        sfConfig::set('app_date_time_pattern_'.$this->getUser()->getUserDomain(), 'yyyy/mm/dd');

        $petrol = mt_rand(0,1);
        $paiddays = mt_rand(0,1);
        $data = [
            'pickup_location_id' => 'owner_'.$branchIdx[0],
            'dropoff_location_id' => 'owner_'.$branchIdx[1],
            'vehicle_type_id' => 'owner_'.$vehicleIdx[0],
            'insurance_included' => true,
            'petrol_included' => $petrol,
            'petrol_included_type' => $petrol,
            'insurance_policy' => $policy->getId(),
            'nb_listings' => mt_rand(1,5),
            'pickup_time' => 3600*8,
            'latest_pickup_time' => 3600*18,
            'dropoff_time' => 3600*8,
            'latest_dropoff_time' => 3600*18,
            'date_available_from' => date('Y/m/d'),
            'date_return_by' => date('Y/m/d', strtotime('+10 days')),
            'days_allowed' => 3,
            'paid_days_allowed' => $paiddays ? 2 : null,
            'price_per_day' => $paiddays ? 30 : null,
            'min_age' => 18
        ];

        $form = new OwnerListingForm(array(), array(
          'country' => $this->getUser()->getCountry(),
          'domain'  => $this->getUser()->getUserDomain(),
          'user'    => $this->getUser()->getAttribute('profile')
        ), false);
        $form->bind($data);

        $form->save($listing = new Listing());

        return $listing;
    }

    private function generateRequests($listing, $nb_requests = null)
    {
        $nb_requests = $nb_requests ?: mt_rand(1, 5);
        $free_days = $listing->get('days_allowed');
        $paid_days = $listing->get('paid_days_allowed');
        $total_days = $free_days+$paid_days;
        $drivers = Doctrine::getTable('User')
            ->createQuery('u')
            ->where('is_owner != ?', 1)
            ->addWhere('user_status = ?', 'active')
            ->orderBy('id desc')
            ->limit(10)
            ->execute();

        $driverIdx = array_rand($drivers->getData(), $nb_requests);

        for ($i=0; $i < $nb_requests; $i++) { 

            $requested_days = mt_rand(1, $total_days);
            // 2 r, 3 f, 2p (5j)
            // 4 r
            $requested_free_days = min($requested_days, $free_days);
            // 2 - 3 = -1 = 0
            // 4 - 3 = 1
            $requested_paid_days = max(0, $requested_free_days - $free_days);
            $request = new Request();
            $request->set('status', 0);
            $request->set('created_at', date('Y-m-d H:i:s'));
            $request->set('Listing', $listing);
            $request->set('Driver', $drivers[$driverIdx[$i]]);
            $request->set('pickup_date', date('Y-m-d', strtotime('+1 day')));
            $request->set('drop_off_date', date('Y-m-d', strtotime('+'.$requested_days.' days')));
            $request->set('pickup_time', 3600*9);
            $request->set('dropoff_time', 3600*9);
            $request->set('nb_free_days', min($free_days, $requested_days));
            $request->set('nb_paid_days', $requested_paid_days);
            $request->save();
        }
    }
}